'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var ResizeObserver = _interopDefault(require('resize-observer-polyfill'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _taggedTemplateLiteralLoose(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  strings.raw = raw;
  return strings;
}

var ApiContext = /*#__PURE__*/React.createContext({});

function useApi(componentName) {
  var apiFromContext = React.useContext(ApiContext);
  return apiFromContext[componentName];
}

function createStyleTag(strings) {
  for (var _len = arguments.length, values = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    values[_key - 1] = arguments[_key];
  }

  var styles = strings.map(function (str, i) {
    return "" + str + (values[i] || '');
  }).join('');
  return React__default.createElement("style", {
    dangerouslySetInnerHTML: {
      __html: styles
    }
  });
}

function createHash(str) {
  var n = 0x5bd1e995;
  var l = str.length,
      h = l ^ l,
      i = 0,
      k;

  while (l >= 4) {
    k = str.charCodeAt(i) & 0xff | (str.charCodeAt(++i) & 0xff) << 8 | (str.charCodeAt(++i) & 0xff) << 16 | (str.charCodeAt(++i) & 0xff) << 24;
    k = (k & 0xffff) * n + (((k >>> 16) * n & 0xffff) << 16);
    k ^= k >>> 24;
    k = (k & 0xffff) * n + (((k >>> 16) * n & 0xffff) << 16);
    h = (h & 0xffff) * n + (((h >>> 16) * n & 0xffff) << 16) ^ k;
    l -= 4;
    ++i;
  }

  switch (l) {
    case 3:
      h ^= (str.charCodeAt(i + 2) & 0xff) << 16;

    case 2:
      h ^= (str.charCodeAt(i + 1) & 0xff) << 8;

    case 1:
      h ^= str.charCodeAt(i) & 0xff;
      h = (h & 0xffff) * 0x5bd1e995 + (((h >>> 16) * 0x5bd1e995 & 0xffff) << 16);
  }

  h ^= h >>> 13;
  h = (h & 0xffff) * n + (((h >>> 16) * n & 0xffff) << 16);
  h ^= h >>> 15;
  return (h >>> 0).toString(36);
}

function enhancePropsWithClassname(props, newClass) {
  var _props$className = props.className,
      className = _props$className === void 0 ? '' : _props$className;
  return _extends({}, props, {
    className: className.length ? className.concat(" " + newClass) : newClass
  });
}
function reduceApi(apiFromDefaults, apiFromContext, props) {
  var apiPrepopulated = _extends({}, apiFromDefaults, {}, apiFromContext);

  return _extends({}, Object.keys(apiPrepopulated).reduce(function (apiFinal, key) {
    if (props.hasOwnProperty(key)) {
      var _extends2;

      var apiValue = props[key];
      return _extends({}, apiFinal, (_extends2 = {}, _extends2[key] = apiValue, _extends2));
    }

    return apiFinal;
  }, apiPrepopulated));
}
function createLayoutClassname(type, api) {
  if (api === void 0) {
    api = {};
  }

  return "layout-block-" + type + "-" + createHash(JSON.stringify(api));
}
function createLayoutConfig(opts) {
  var defaultApi = opts.defaultApi,
      contextApi = opts.contextApi,
      name = opts.name,
      props = opts.props;

  var _props$as = props.as,
      Tag = _props$as === void 0 ? 'div' : _props$as,
      children = props.children,
      remainingProps = _objectWithoutPropertiesLoose(props, ["as", "children"]);

  var api = reduceApi(defaultApi, contextApi, remainingProps);
  var layoutClass = createLayoutClassname(name, api);
  var selector = Tag + "." + layoutClass;
  return {
    api: api,
    children: children,
    passedProps: enhancePropsWithClassname(remainingProps, layoutClass),
    selector: selector,
    Tag: Tag
  };
}

function _templateObject() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          padding: ", ";\n        }\n      "]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi = {
  padding: 'var(--s1)'
};
var name = 'box';
/**
 * Box layout component
 */

function Box(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name),
    defaultApi: defaultApi,
    name: name,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var padding = api.padding;
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject(), selector, padding), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

function _templateObject$1() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          box-sizing: content-box;\n          margin-left: auto;\n          margin-right: auto;\n          max-width: ", ";\n          ", "\n          padding-left: ", ";\n          padding-right: ", ";\n          ", "\n        }\n      "]);

  _templateObject$1 = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$1 = {
  max: 'var(--measure)',
  centerText: false,
  gutter: '0',
  isIntrinsic: false
};
var name$1 = 'center';
/**
 * Center layout component
 */

function Center(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$1),
    defaultApi: defaultApi$1,
    name: name$1,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var max = api.max,
      centerText = api.centerText,
      gutter = api.gutter,
      intrinsic = api.intrinsic;
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$1(), selector, max, centerText ? 'text-align: center;' : '', gutter, gutter, intrinsic ? "\n            display:flex;\n            flex-direction: column;\n            align-items: center;\n          " : ''), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

function _templateObject$2() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          display: flex;\n          flex-direction: column;\n          min-height: ", ";\n          padding: ", ";\n        }\n  \n        ", " > * {\n          margin-top: ", ";\n          margin-bottom: ", ";\n        }\n  \n        ", " {\n          display: flex;\n          flex-direction: column;\n          min-height: ", ";\n          padding: ", ";\n        }\n  \n        ", " > :first-of-type:not(style) {\n          margin-top: 0;\n        }\n  \n        ", " > :last-of-type:not(style) {\n          margin-bottom: 0;\n        }\n  \n        ", "\n      "]);

  _templateObject$2 = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$2 = {
  centeredIndex: 2,
  space: 'var(--s1)',
  minHeight: '100vh',
  isPadded: false
};
var name$2 = 'cover';
/**
 * Cover layout component
 */

function Cover(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$2),
    defaultApi: defaultApi$2,
    name: name$2,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var centeredIndex = api.centeredIndex,
      space = api.space,
      minHeight = api.minHeight,
      isPadded = api.isPadded;
  var childrenCount = React.Children.count(children);
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$2(), selector, minHeight, isPadded ? space : '0', selector, space, space, selector, minHeight, isPadded ? space : '0', selector, selector, childrenCount === 1 ? "\n          " + selector + " > :first-of-type:not(style) {\n            margin-top: auto;\n            margin-bottom: auto;\n          }\n        " : "\n        " + selector + " > :nth-of-type(" + centeredIndex + "):not(style) {\n          margin-top: auto;\n          margin-bottom: auto;\n        }\n        "), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

function _templateObject$3() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          --n: ", ";\n          --d: ", ";\n          padding-bottom: calc(var(--n) / var(--d) * 100%);\n          position: relative;\n        }\n  \n        ", " > * {\n          overflow: hidden;\n          position: absolute;\n          top: 0;\n          right: 0;\n          bottom: 0;\n          left: 0;\n          display: flex;\n          justify-content: center;\n          align-items: center;\n        }\n  \n        ", " > img,\n        ", " > video {\n          width: 100%;\n          height: 100%;\n          object-fit: cover;\n        }\n      "]);

  _templateObject$3 = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$3 = {
  ratio: '16:9'
};
var name$3 = 'frame';
/**
 * Frame layout component
 */

function Frame(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$3),
    defaultApi: defaultApi$3,
    name: name$3,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var ratio = api.ratio;

  var _ratio$split = ratio.split(':'),
      denominator = _ratio$split[0],
      numerator = _ratio$split[1];

  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$3(), selector, Number(numerator), Number(denominator), selector, selector, selector), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

function _templateObject$4() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          width: 100vw;\n          margin-left: 50%;\n          transform: translateX(-50%);\n        }\n      "]);

  _templateObject$4 = function _templateObject() {
    return data;
  };

  return data;
}
var name$4 = 'breakout';
/**
 * Breakout layout component
 */

function Breakout(props) {
  var _createLayoutConfig = createLayoutConfig({
    name: name$4,
    props: props
  }),
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$4(), selector), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

var IS_BROWSER = typeof window !== 'undefined';
/**
 * Watch for the resizing of a React component or Element.
 *
 * @param hookProperties - Configuration optinos for the hook.
 *
 * @returns The `DOMRect` for the observed element.
 */

var useResizeObserver = function useResizeObserver(_ref) {
  var ref = _ref.ref,
      element = _ref.element,
      callback = _ref.callback;

  var _useState = React.useState({
    bottom: 0,
    height: 0,
    left: 0,
    right: 0,
    top: 0,
    width: 0,
    x: 0,
    y: 0
  }),
      sizes = _useState[0],
      setSizes = _useState[1];

  var handleResize = function handleResize(entries) {
    var entry = entries[0];
    if (callback) callback(entry);
    setSizes(entry.contentRect);
  };

  var _useState2 = React.useState(function () {
    return IS_BROWSER ? new ResizeObserver(handleResize) : undefined;
  }),
      resizeObs = _useState2[0];

  React.useEffect(function () {
    if (!resizeObs) return;
    var domNode;
    if (ref) domNode = ref.current;else if (element) domNode = element;
    if (domNode) resizeObs.observe(domNode);
    return function () {
      return resizeObs.disconnect();
    };
  }, [ref, resizeObs, element]);
  return sizes;
};

function _templateObject$5() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          display: grid;\n          grid-gap: ", ";\n          grid-template-columns: 100%;\n        }\n  \n        ", ".aboveMin {\n          grid-template-columns: repeat(auto-fill, minmax(", ", 1fr));\n        }\n  \n        @supports (width: min(", ", 100%)) {\n          ", " {\n            grid-template-columns: repeat(auto-fit, minmax(min(", ", 100%), 1fr));\n          }\n        }\n      "]);

  _templateObject$5 = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$4 = {
  min: '250px',
  space: 'var(--s1)'
};
var name$5 = 'grid';
/**
 * Grid layout component
 */

function Grid(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$5),
    defaultApi: defaultApi$4,
    name: name$5,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var min = api.min,
      space = api.space;
  var ref = React.useRef(null);
  var childRef = React.useRef(null);

  var _useResizeObserver = useResizeObserver({
    ref: ref
  }),
      width = _useResizeObserver.width;

  var isWide = false; // Add a test element to compare the min value to the container

  if (childRef.current) {
    childRef.current.style.width = min;
    isWide = width > childRef.current.offsetWidth;
  }

  var containerClass = 'aboveMin';
  var className = passedProps.className;

  var enhancedProps = _extends({}, passedProps, {
    ref: ref,
    className: className.length ? isWide ? className.concat(" " + containerClass) : className : isWide ? containerClass : className
  });

  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$5(), selector, space, selector, min, min, selector, min), React__default.createElement(Tag, Object.assign({}, enhancedProps), children, React__default.createElement("div", {
    ref: childRef
  })));
}

function _templateObject$6() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          overflow: hidden;\n        }\n  \n        ", " > * {\n          display: flex;\n          flex-wrap: wrap;\n          justify-content: ", ";\n          align-items: ", ";\n          margin: calc(", " / 2 * -1);\n        }\n  \n        ", " > * > * {\n          margin: calc(", " / 2);\n        }\n      "]);

  _templateObject$6 = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$5 = {
  align: 'center',
  justify: 'start',
  space: 'var(--s1)'
};
var name$6 = 'group';
/**
 * Group layout component
 */

function Group(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$6),
    defaultApi: defaultApi$5,
    name: name$6,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var align = api.align,
      justify = api.justify,
      space = api.space;
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$6(), selector, selector, ['start', 'end'].includes(justify) ? "flex-" + justify : justify, ['start', 'end'].includes(align) ? "flex-" + align : align, space, selector, space), React__default.createElement(Tag, Object.assign({}, passedProps), React__default.createElement("div", null, children)));
}

function _templateObject$7() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          ", " {\n            position: absolute;\n            top: 50%;\n            left: 50%;\n            transform: translate(-50%, -50%);\n            ", "\n            ", "\n          }\n        }\n      "]);

  _templateObject$7 = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$6 = {
  breakout: false,
  margin: '0px',
  isFixed: false
};
var name$7 = 'imposter';
/**
 * Imposter layout component
 */

function Imposter(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$7),
    defaultApi: defaultApi$6,
    name: name$7,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var breakout = api.breakout,
      margin = api.margin,
      fixed = api.fixed;
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$7(), selector, selector, !breakout ? "\n              max-width: calc(100% - (" + margin + " * 2));\n              max-height: calc(100% - (" + margin + " * 2));\n              overflow: auto;" : '', fixed ? "\n              position: fixed;" : ''), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

function _templateObject$8() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          overflow: hidden;\n        }\n  \n        ", " > * {\n          display: flex;\n          flex-wrap: wrap;\n          ", "\n          margin: calc(", " / 2 * -1);\n        }\n  \n        ", " > * > * {\n          margin: calc(", " / 2);\n          ", "\n          flex-grow: 1;\n        }\n  \n        ", " > * > :", ":not(style) {\n          flex-basis: 0;\n          flex-grow: 999;\n          min-width: calc(", " - ", ");\n        }\n      "]);

  _templateObject$8 = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$7 = {
  side: 'left',
  sideWidth: null,
  contentMin: '50%',
  space: 'var(--s1)',
  isStretch: false
};
var name$8 = 'sidebar';
/**
 * Sidebar layout component
 */

function Sidebar(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$8),
    defaultApi: defaultApi$7,
    name: name$8,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var side = api.side,
      sideWidth = api.sideWidth,
      contentMin = api.contentMin,
      space = api.space,
      isStretch = api.isStretch;
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$8(), selector, selector, !isStretch ? "align-items: flex-start;" : '', space, selector, space, sideWidth ? "flex-basis: " + sideWidth + ";" : '', selector, side === 'left' ? 'last-of-type' : 'first-of-type', contentMin, space), React__default.createElement(Tag, Object.assign({}, passedProps), React__default.createElement("div", null, children)));
}

function _templateObject$9() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          flex: 1 0 auto;\n        }\n      "]);

  _templateObject$9 = function _templateObject() {
    return data;
  };

  return data;
}
var name$9 = 'spacer';
/**
 * Spacer layout component
 */

function Spacer(props) {
  var _createLayoutConfig = createLayoutConfig({
    name: name$9,
    props: props
  }),
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$9(), selector), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

function _templateObject$a() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " {\n          display: flex;\n          flex-direction: column;\n          justify-content: flex-start;\n        }\n    \n        ", " ", " *:not(style) ~ *:not(style) {\n          margin-top: ", ";\n        }\n      \n        ", "\n      "]);

  _templateObject$a = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$8 = {
  recursive: false,
  splitAfter: null,
  space: 'var(--s1)'
};
var name$a = 'stack';
/**
 * Stack layout component
 */

function Stack(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$a),
    defaultApi: defaultApi$8,
    name: name$a,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var recursive = api.recursive,
      space = api.space,
      splitAfter = api.splitAfter;
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$a(), selector, selector, !recursive ? '>' : '', space, splitAfter ? "\n            " + selector + ":only-child {\n            height: 100%;\n          }\n          " + selector + " > :nth-child(" + splitAfter + ") {\n            margin-bottom: auto;\n          }\n        " : ''), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

function _templateObject$b() {
  var data = _taggedTemplateLiteralLoose(["\n        ", " > * {\n          display: flex;\n          flex-wrap: wrap;\n          margin: calc((", " / 2) * -1);\n        }\n  \n        ", " > * > * {\n          flex-grow: 1;\n          flex-basis: calc((", " - (100% - ", ")) * 999);\n          margin: calc(", " / 2);\n        }\n  \n        ", " > * > :nth-last-of-type(n+", "):not(style),\n        ", " > * > :nth-last-of-type(n+", "):not(style) ~ *:not(style) {\n          flex-basis: 100%;\n        }\n      "]);

  _templateObject$b = function _templateObject() {
    return data;
  };

  return data;
}
var defaultApi$9 = {
  threshold: 'var(--measure)',
  space: 'var(--s1)',
  limit: 4
};
var name$b = 'switcher';
/**
 * Switcher layout component
 */

function Switcher(props) {
  var _createLayoutConfig = createLayoutConfig({
    contextApi: useApi(name$b),
    defaultApi: defaultApi$9,
    name: name$b,
    props: props
  }),
      api = _createLayoutConfig.api,
      children = _createLayoutConfig.children,
      Tag = _createLayoutConfig.Tag,
      passedProps = _createLayoutConfig.passedProps,
      selector = _createLayoutConfig.selector;

  var threshold = api.threshold,
      space = api.space,
      limit = api.limit;
  return React__default.createElement(React__default.Fragment, null, createStyleTag(_templateObject$b(), selector, space, selector, threshold, space, space, selector, Number(limit) + 1, selector, Number(limit) + 1), React__default.createElement(Tag, Object.assign({}, passedProps), children));
}

exports.Box = Box;
exports.Breakout = Breakout;
exports.Center = Center;
exports.Cover = Cover;
exports.Frame = Frame;
exports.Grid = Grid;
exports.Group = Group;
exports.Imposter = Imposter;
exports.Sidebar = Sidebar;
exports.Spacer = Spacer;
exports.Stack = Stack;
exports.Switcher = Switcher;
//# sourceMappingURL=layout-blocks.cjs.development.js.map
