import { RefObject } from 'react';
interface DOMRectReadOnly {
    readonly bottom: number;
    readonly height: number;
    readonly left: number;
    readonly right: number;
    readonly top: number;
    readonly width: number;
    readonly x: number;
    readonly y: number;
}
/**
 * Hook parameters.
 */
interface useResizeObserverProperties {
    /**
     * Ref object from `useRef`.
     */
    ref?: RefObject<Element> | null;
    /**
     * DOM element. E.g. from `querySelector()`
     */
    element?: Element | null | undefined;
    /**
     * Callback to fire when the observed component or Element
     * resizes.
     */
    callback?: (entry: ResizeObserverEntry) => void;
}
/**
 * Watch for the resizing of a React component or Element.
 *
 * @param hookProperties - Configuration optinos for the hook.
 *
 * @returns The `DOMRect` for the observed element.
 */
declare const useResizeObserver: ({ ref, element, callback, }: useResizeObserverProperties) => DOMRectReadOnly;
export default useResizeObserver;
